#****************************************
# Casini-Kurve (2D)
#***************************************

from lib.cadmath import *

def cassini_func(x,y,a=1.1,c=1.0):
    return (x*x+y*y)*(x*x+y*y)-2*c*c*(x*x-y*y)-(a*a*a*a-c*c*c*c)

def cassini_grad(x,y,c=1.0):
    return (float(4.0*x*(x*x+y*y)-4.0*c*c*x),float(4.0*y*(x*x+y*y)+4.0*c*c*y))

def curve_point(func,grad,v2d,delta):
#Berechnet einen Punkt der Kurve entlang des "steilsten Weges"
    x=v2d.x
    y=v2d.y
    f=func(x,y)
    gx,gy=grad(x,y)
    cc=gx*gx+gy*gy
    if cc>0: f=-f/cc
    else: f=0
    newv2d=Vector2D(x+f*gx,y+f*gy)
    d=v2d.sub(newv2d).length()
    while delta<d:
        v2d=newv2d
        x=v2d.x
        y=v2d.y
        f=func(x,y)
        gx,gy=grad(x,y)
        cc=gx*gx+gy*gy
        if cc>0: f=-f/cc
        else: f=0
        newv2d=Vector2D(x+f*gx,y+f*gy)
        d=v2d.sub(newv2d).length()
    return newv2d

def implicit_curve_points(func,grad,v2d0,v2d1,maxpoints,step,delta):
#Berechnet Kurvenpunkte einer impliziten Kurve mit Hilfe von curve_point
    pointlist=[]
    startv2d=curve_point(func,grad,v2d0,delta)
    pointlist.append(startv2d)
    endv2d=curve_point(func,grad,v2d1,delta)
    gx,gy=grad(startv2d.x,startv2d.y)
    tangentv2d=Vector2D(-gy,gx)
    tangentlen=tangentv2d.length()
    startv2d=curve_point(func,grad,startv2d.add(tangentv2d.scale(step/tangentlen)),delta)
    pointlist.append(startv2d)
    distance=step
    while (distance>step/2.0) and (len(pointlist)<maxpoints):
        gx,gy=grad(startv2d.x,startv2d.y)
        tangentv2d=Vector2D(-gy,gx)
        tangentlen=tangentv2d.length()
        startv2d=curve_point(func,grad,startv2d.add(tangentv2d.scale(step/tangentlen)),delta)
        pointlist.append(startv2d)
        distance=startv2d.distance(endv2d)
    pointlist.append(endv2d)
    return pointlist

pointlist=implicit_curve_points(cassini_func,cassini_grad,Vector2D(2,0),Vector2D(2,0),100,0.2,0.001)
graph.setScaling(100,100)
graph.drawCurve2D(pointlist)
for point in pointlist:
    graph.drawPoint2D(point,"red",3)
