import math


class Vector2D:

    def __init__(self,x=0,y=0):
        self.x=x
        self.y=y

    def toString(self): return "["+str(self.x)+";"+str(self.y)+"]"

    def equals(self,v2d): return self.x==v2d.x and self.y==v2d.y

    def add(self,v2d): return Vector2D(self.x+v2d.x,self.y+v2d.y)

    def sub(self,v2d): return Vector2D(self.x-v2d.x,self.y-v2d.y)
    
    def scalarProduct(self,v2d): return self.x*v2d.x+self.y*v2d.y

    def scale(self,f): return Vector2D(self.x*f,self.y*f)

    def scaleCoords(self,fx,fy): return Vector2D(self.x*fx,self.y*fy)

    def abs(self): return Vector2D(abs(self.x),abs(self.y))

    def length(self): return float(math.sqrt(self.x*self.x+self.y*self.y))

    def normalize(self): return Vector2D(float(self.x)/self.length(),float(self.y)/self.length())

    def squareDistance(self,v2d): return math.pow(self.x-v2d.x,2)+math.pow(self.y-v2d.y,2)

    def distance(self,v2d): return float(math.sqrt(self.squareDistance(v2d)))

    def rotate(self,deg):
        sin,cos=function_DegToSinCos(deg)
        return Vector2D(self.x*cos-self.y*sin,self.x*sin+self.y*cos)
        
    def rotateAroundPoint(self,v2d,deg):
        sin,cos=function_DegToSinCos(deg)
        v2d0=self.sub(v2d)
        return Vector2D(v2d0.x*cos-v2d0.y*sin,v2d0.x*sin+v2d0.y*cos).add(v2d)

    def polarCoords(self,v2d): # Berechnet die Polarkoordinaten des Punktes (x,y)
        half_pi=math.pi*0.5
        x=v2d.x
        y=v2d.y
        if x==0:
            rho=y
            phi=0
            if y<0: phi=half_pi*3
            if y>0: phi=half_pi
        else:
            rho=math.sqrt(x*x+y*y)
            h=float(y)/x
            phi=math.atan(h)
            if h<0: phi+=half_pi
            if y<0: phi+=half_pi
        return(rho,phi)


class Vector3D:

    def __init__(self,x=0,y=0,z=0):
        self.x=x
        self.y=y
        self.z=z

    def toString(self): return "["+str(self.x)+";"+str(self.y)+";"+str(self.z)+"]"

    def equals(self,v3d): return self.x==v3d.x and self.y==v3d.y and self.z==v3d.z

    def add(self,v3d): return Vector3D(self.x+v3d.x,self.y+v3d.y,self.z+v3d.z)

    def sub(self,v3d): return Vector3D(self.x-v3d.x,self.y-v3d.y,self.z-v3d.z)

    def scalarProduct(self,v3d): return self.x*v3d.x+self.y*v3d.y+self.z*v3d.z

    def scale(self,f): return Vector3D(self.x*f,self.y*f,self.z*f)

    def scaleCoords(self,fx,fy,fz): return Vector3D(self.x*fx,self.y*fy,self.z*fz)

    def abs(self): return Vector3D(abs(self.x),abs(self.y),abs(self.z))

    def length(self): return math.sqrt(self.x*self.x+self.y*self.y+self.z*self.z)

    def normalize(self): return Vector3D(float(self.x)/self.length(),float(self.y)/self.length(),float(self.z)/self.length())

    def squareDistance(self,v3d): return math.pow(self.x-v3d.x,2)+math.pow(self.y-v3d.y,2)+math.pow(self.z-v3d.z,2)

    def distance(self,v3d): return math.sqrt(self.squareDistance(v3d))

    def vectorProduct(self,v3d): return Vector3D(self.y*v3d.z-self.z*v3d.y,self.z*v3d.x-self.x*v3d.z,self.x*v3d.y-self.y*v3d.x)

    def rotateX(self,deg):
        sin,cos=function_DegToSinCos(deg)
        return Vector3D(self.x,self.y*cos-self.z*sin,self.y*sin+self.z*cos)

    def rotateY(self,deg):
        sin,cos=function_DegToSinCos(deg)
        return Vector3D(self.x*cos-self.z*sin,self.y,self.x*sin+self.z*cos)

    def rotateZ(self,deg):
        sin,cos=function_DegToSinCos(deg)
        return Vector3D(self.x*cos-self.y*sin,self.x*sin+self.y*cos,self.z)
                
    def rotateXAroundPoint(self,v3d,deg):
        sin,cos=function_DegToSinCos(deg)
        v3d0=self.sub(v3d)
        return Vector3D(v3d.x,v3d0.y*cos-v3d0.z*sin+v3d.y,v3d0.y*sin+v3d0.z*cos+v3d.z)

    def rotateYAroundPoint(self,v3d,deg):
        sin,cos=function_DegToSinCos(deg)
        v3d0=self.sub(v3d)
        return Vector3D(v3d0.x*cos-v3d0.z*sin+v3d.x,v3d.y,v3d0.x*sin+v3d0.z*cos+v3d.z)

    def rotateZAroundPoint(self,v3d,deg):
        sin,cos=function_DegToSinCos(deg)
        v3d0=self.sub(v3d)
        return Vector3D(v3d0.x*cos-v3d0.y*sin+v3d.x,v3d0.x*sin+v3d0.y*cos+v3d.y,v3d.z)

    def newCoords(self,v3d0,v3dx,v3dy,v3dz): #{Berechnet die Koordinaten von p bzgl. der Basis v3dx,v3dy,v3dz mit Nullpkt. v3d0.}
        h0=self.sub(v3d0)
        h1=1.0/Matrix3x3(v3dx,v3dy,v3dz).determinant()
        return Vector3D(Matrix3x3(h0,v3dy,v3dz).determinant(),Matrix3x3(v3dx,h0,v3dz).determinant(),Matrix3x3(v3dx,v3dy,h0).determinant()).scale(h1)


class Matrix3x3:

	def __init__(self,v3d0=None,v3d1=None,v3d2=None):
		if v3d0==None: self.x0,self.y0,self.z0=0,0,0
		else: self.x0,self.y0,self.z0=v3d0.x,v3d0.y,v3d0.z
		if v3d1==None: self.x1,self.y1,self.z1=0,0,0
		else: self.x1,self.y1,self.z1=v3d1.x,v3d1.y,v3d1.z
		if v3d2==None: self.x2,self.y2,self.z2=0,0,0
		else: self.x2,self.y2,self.z2=v3d2.x,v3d2.y,v3d2.z

	def determinant(self):
		x0,y0,z0,x1,y1,z1,x2,y2,z2=self.x0,self.y0,self.z0,self.x1,self.y1,self.z1,self.x2,self.y2,self.z2
		return x0*y1*z2+x1*y2*z0+x2*y0*z1-z0*y1*x2-z1*y2*x0-z2*y0*x1


class Node3D:
    
    def __init__(self,v3d):
        self.v3d=v3d
        self.extension=None


class Edge3D: # Kante - verwendbar fr 3D-Vektoren

    def __init__(self,startnode,endnode):
        self.startnode=startnode
        self.endnode=endnode
        self.extension=None

class Face3D: # Die Lnge der nodelist muss >2 sein damit Flaeche

    def __init__(self,nodelist,edgelist):
        self.nodelist=nodelist
        self.edgelist=edgelist
        self.extension=None


class Object3D:

    def __init__(self):
        self.nodelist=[] #beinhaltet Node3D's
        self.edgelist=[] #beinhaltet Edge3D's
        self.facelist=[] #beinhaltet Face3D's

    def merge(self,obj3D):
        retobj3D=Object3D()
        retobj3D.nodelist[:0]=obj3D.nodelist
        retobj3D.nodelist[:0]=self.nodelist
        retobj3D.edgelist[:0]=obj3D.edgelist
        retobj3D.edgelist[:0]=self.edgelist
        retobj3D.facelist[:0]=obj3D.facelist
        retobj3D.facelist[:0]=self.facelist
        return retobj3D

    def rotateX(self,deg):
        for node in self.nodelist: node.v3d=node.v3d.rotateX(deg)

    def rotateY(self,deg):
        for node in self.nodelist: node.v3d=node.v3d.rotateY(deg)

    def rotateZ(self,deg):
        for node in self.nodelist: node.v3d=node.v3d.rotateZ(deg)

    def translate(self,x,y,z):
        for node in self.nodelist: node.v3d=node.v3d.add(Vector3D(x,y,z))


class Cuboid(Object3D):

    def __init__(self,xlen=10,ylen=10,zlen=10):
        Object3D.__init__(self)
        halfx=xlen*0.5
        halfy=ylen*0.5
        halfz=zlen*0.5
        nodelist=self.nodelist
        edgelist=self.edgelist
        facelist=self.facelist
        nodelist.append(Node3D(Vector3D(halfx,halfy,halfz)))
        nodelist.append(Node3D(Vector3D(-halfx,halfy,halfz)))
        nodelist.append(Node3D(Vector3D(-halfx,-halfy,halfz)))
        nodelist.append(Node3D(Vector3D(halfx,-halfy,halfz)))
        nodelist.append(Node3D(Vector3D(halfx,halfy,-halfz)))
        nodelist.append(Node3D(Vector3D(halfx,-halfy,-halfz)))
        nodelist.append(Node3D(Vector3D(-halfx,-halfy,-halfz)))
        nodelist.append(Node3D(Vector3D(-halfx,halfy,-halfz)))
        edgelist.append(Edge3D(nodelist[0],nodelist[1]))
        edgelist.append(Edge3D(nodelist[1],nodelist[2]))
        edgelist.append(Edge3D(nodelist[3],nodelist[2]))
        edgelist.append(Edge3D(nodelist[0],nodelist[3]))
        edgelist.append(Edge3D(nodelist[4],nodelist[5]))
        edgelist.append(Edge3D(nodelist[5],nodelist[6]))
        edgelist.append(Edge3D(nodelist[7],nodelist[6]))
        edgelist.append(Edge3D(nodelist[4],nodelist[7]))
        edgelist.append(Edge3D(nodelist[4],nodelist[0]))
        edgelist.append(Edge3D(nodelist[7],nodelist[1]))
        edgelist.append(Edge3D(nodelist[6],nodelist[2]))
        edgelist.append(Edge3D(nodelist[5],nodelist[3]))
        facelist.append(Face3D((nodelist[:4]),(edgelist[:4])))
        facelist.append(Face3D((nodelist[4:]),(edgelist[4:8])))
        facelist.append(Face3D((nodelist[4],nodelist[7],nodelist[1],nodelist[0]),(edgelist[7],edgelist[9],edgelist[0],edgelist[8])))
        facelist.append(Face3D((nodelist[4],nodelist[0],nodelist[3],nodelist[5]),(edgelist[8],edgelist[3],edgelist[11],edgelist[4])))
        facelist.append(Face3D((nodelist[5],nodelist[3],nodelist[2],nodelist[6]),(edgelist[11],edgelist[2],edgelist[10],edgelist[5])))
        facelist.append(Face3D((nodelist[7],nodelist[6],nodelist[2],nodelist[1]),(edgelist[6],edgelist[10],edgelist[1],edgelist[9])))


class Cube(Cuboid):

    def __init__(self,len):
        Cuboid.__init__(self,len,len,len)


class Cylinder(Object3D):

    def __init__(self,diameter=10,height=10,partitions=6):
        Object3D.__init__(self)
        nodelist=self.nodelist
        edgelist=self.edgelist
        facelist=self.facelist
        neghalfheight=height*-0.5
        startangle=360.0/partitions
        startpoint=Vector3D(diameter*0.5,neghalfheight,0)
        for i in range(partitions): nodelist.append(Node3D(startpoint.rotateY(startangle*i)))
        for node in nodelist[:partitions]: nodelist.append(Node3D(node.v3d.scaleCoords(1,-1,-1)))
        node0=nodelist[0]
        node1=nodelist[partitions]
        node2=nodelist[-1]
        node3=nodelist[1]
        edgelist.append(Edge3D(node0,node1))
        edgelist.append(Edge3D(node0,node3))
        edgelist.append(Edge3D(node2,node1))
        edgelist.append(Edge3D(node3,node2))
        facelist.append(Face3D((node0,node1,node2,node3),edgelist[0:]))
        for i in range(1,partitions-1):
            node0=nodelist[i]
            node1=nodelist[-i]
            node2=nodelist[-i-1]
            node3=nodelist[i+1]
            edgelist.append(Edge3D(node0,node3))
            edgelist.append(Edge3D(node2,node1))
            edgelist.append(Edge3D(node3,node2))
            facelist.append(Face3D((node0,node1,node2,node3),edgelist[-4:]))
        node0=nodelist[partitions-1]
        node1=nodelist[partitions+1]
        node2=nodelist[partitions]
        node3=nodelist[0]
        edgelist.append(Edge3D(node0,node3))
        edgelist.append(Edge3D(node2,node1))
        facelist.append(Face3D((node0,node1,node2,node3),edgelist[-3:]+edgelist[0:1]))
        helpedgelist=[]
        for i in range(1,len(edgelist),3): helpedgelist.append(edgelist[i])
        facelist.append(Face3D(nodelist[:partitions],helpedgelist))
        helpedgelist=[]
        for i in range(2,len(edgelist),3): helpedgelist.append(edgelist[i])
        facelist.append(Face3D(nodelist[partitions:],helpedgelist))


class Torus(Object3D):

    def __init__(self,diameter=10,tubediameter=5,partitions=6,tubepartitions=6):
        Object3D.__init__(self)
        nl=self.nodelist
        el=self.edgelist
        fl=self.facelist
        startangle=360.0/partitions
        tubestartangle=360.0/tubepartitions
        startpoint=Vector3D(diameter*0.5,0,0)
        tubestartpoint=Vector3D((diameter+tubediameter)*0.5,0,0)
        for i in range(partitions):
            moveangle=startangle*i
            for j in range(tubepartitions): nl.append(Node3D(tubestartpoint.rotateZAroundPoint(startpoint,tubestartangle*j).rotateY(moveangle)))
        tpsub1=tubepartitions-1
        el.append(Edge3D(nl[-tubepartitions],nl[0]))
        el.append(Edge3D(nl[0],nl[tpsub1]))
        for j in range(tpsub1):
            el.append(Edge3D(nl[-tpsub1+j],nl[j+1]))
            el.append(Edge3D(nl[j+1],nl[j]))
        for i in range(partitions-1):
            tpmuli=tubepartitions*i
            el.append(Edge3D(nl[tpmuli],nl[tpmuli+tubepartitions]))
            el.append(Edge3D(nl[tpmuli+tubepartitions],nl[tpmuli+2*tubepartitions-1]))
            for j in range(tpsub1):
                el.append(Edge3D(nl[tpmuli+j+1],nl[tpmuli+tubepartitions+j+1]))
                el.append(Edge3D(nl[tpmuli+tubepartitions+j+1],nl[tpmuli+tubepartitions+j]))
                fl.append(Face3D((nl[tpmuli+j],nl[tpmuli+j+1],nl[tpmuli+tubepartitions+j+1],nl[tpmuli+tubepartitions+j]),(el[-2*tubepartitions-1],el[-2],el[-1],el[-4])))
            fl.append(Face3D((nl[tpmuli+tubepartitions-1],nl[tpmuli],nl[tpmuli+tubepartitions],nl[tpmuli+2*tubepartitions-1]),(el[-4*tubepartitions+1],el[-2*tubepartitions],el[-2*tubepartitions+1],el[-2])))
        for j in range(tpsub1):
            fl.append(Face3D((nl[-tubepartitions+j],nl[-tubepartitions+j+1],nl[j+1],nl[j]),(el[2*(-tpsub1+j)+1],el[2*j+2],el[2*j+3],el[2*j])))
        fl.append(Face3D((nl[-1],nl[-tubepartitions],nl[0],nl[tubepartitions-1]),(el[-2*tubepartitions+1],el[0],el[1],el[2*tubepartitions-2])))


# Intersection:

def intersect2Lines(a1,b1,c1,a2,b2,c2): # line:=ax+by=c; divide zero exception will be raised
    det=float(a1*b2-a2*b1)
    return ((c1*b2-c2*b1)/det,(a1*c2-a2*c1)/det)

def intersectLineCircle(a,b,c,xm,ym,r): # Exception wenn kein Schnittpunkt, 1SP->Tupel mit gleichen Werten
    h0=a*a+b*b
    h1=c-a*xm-b*ym
    h2=math.sqrt(r*r*h0-h1*h1)
    x1=xm+float(a*h1-b*h2)/h0
    y1=ym+float(b*h1+a*h2)/h0
    x2=xm+float(a*h1+b*h2)/h0
    y2=ym+float(b*h1-a*h2)/h0
    if x1>x2: return (Vector2D(x2,y2),Vector2D(x1,y1))
    return (Vector2D(x1,y1),Vector2D(x2,y2))

def intersectLineUnitcircle(a,b,c,r): # Exception wenn kein Schnittpunkt, 1SP->Tupel mit gleichen Werten
    return intersectLineCircle(a,b,c,0,0,r)

def intersectLinePlane(v3d,r_v3d,n_v3d,d): #Exception wenn kein schnitt oder gerade in ebene
    h0=n_v3d.scalarProduct(r_v3d)
    h1=n_v3d.scalarProduct(v3d)-d
    if h0: return function_linearCombine3D(v3d,1,r_v3d,float(-h1)/h0)
    raise "There is no intersection point" #schnitt kein Punkt
                            
def intersect2Circles(xm0,ym0,r0,xm1,ym1,r1): # Exception wenn kein Schnittpunkt, 1SP->Tupel mit gleichen Werten
    h0=2*(xm1-xm0)
    h1=2*(ym1-ym0)
    h2=r0*r0-xm0*xm0-ym0*ym0-r1*r1+xm1*xm1+ym1*ym1
    return intersectLineCircle(h0,h1,h2,xm0,ym0,r0)

def intersect2Planes(n_v3d0,d0,n_v3d1,d1): #Exception wenn schnitt keine gerade 
    h0=n_v3d0.scalarProduct(n_v3d0)
    h1=n_v3d1.scalarProduct(n_v3d1)
    h2=n_v3d0.scalarProduct(n_v3d1)
    h3=h0*h1-h2*h2
    if h3: return(function_linearCombine3D(n_v3d0,float(d0*h1-d1*h2)/h3,n_v3d1,float(d1*h0-d0*h2)/h3),n_v3d0.vectorPoduct(n_v3d1))
    raise "There is no line"#schnitt keine gerade

def intersect3Planes(n_v3d0,d0,n_v3d1,d1,n_v3d2,d2): #Exception wenn schnitt kein Punkt
    h0=n_v3d0.vectorProduct(n_v3d1)
    h1=n_v3d1.vectorProduct(n_v3d2)
    h2=n_v3d2.vectorProduct(n_v3d0)
    h3=n_v3d0.scalarProduct(h1)
    if h3: return function_linearCombine3D(float(d0)/h3,h1,float(d1)/h3,h2,float(d2)/h3,h0)
    raise #schnitt kein Punkt


# Function:

def function_linearCombine2D(v2d0,r0,v2d1,r1,v2d2=None,r2=0):
    if v2d2==None: v2d2=Vector()
    return Vector2D(r0*v2d0.x+r1*v2d1.x+r2*v2d2.x,r0*v2d0.y+r1*v2d1.y+r2*v2d2.y)

def function_linearCombine3D(v3d0,r0,v3d1,r1,v3d2=None,r2=0):
    if v3d2==None: v3d2=Vector3D()
    return Vector3D(r0*v3d0.x+r1*v3d1.x+r2*v3d2.x,r0*v3d0.y+r1*v3d1.y+r2*v3d2.y,r0*v3d0.z+r1*v3d1.z+r2*v3d2.z)

def function_DegToSinCos(deg):
    deg=deg*math.pi/180
    return (math.sin(deg),math.cos(deg))

def function_SolveEquationDeg1(a,b): return float(-b)/a  # divide zero exception will be raised

def function_SolveEquationDeg2(a,b,c): # sqrt exception will be raised
    if a: # wenn a!=0 
        h0=math.sqrt(float(b*b-4*a*c)/4*a*a)
        h1=float(b)/-2*a
        return (h1+h0,h1-h0)
    return function_SolveEquationDeg1(b,c)

def function_PointOverPlane(v3d,n_v3d,d): #{...stellt fest, ob der Punkt p "vor" der Ebene  nv*x-d=0 liegt.}
    return v3d.scalarProduct(n_v3d)-d>=0

def function_GetPlaneEquation(v3d0,v3d1,v3d2): #Exception wenn keine ebene
    n_v3d=((v3d1.sub(v3d0)).vectorProduct(v3d2.sub(v3d0))).normalize()
    if not n_v3d.equals(Vector3D()): return (n_v3d,n_v3d.scalarProduct(v3d0))
    raise

def function_SolvePlaneEquation(v3d,r1_v3d,r2_v3d,p_v3d): #Exception
    # {v1,v2 sind linear unabhaengig, p-p0 linear abhaengig von v1,v2.
    # Es werden Zahlen  xi,eta berechnet mit  p = p0 + xi*v1 + eta*v2. }
        s00=r_v3d0.scalarProduct(r_v3d0)
        s11=r_v3d1.scalarProduct(r_v3d1)
        s01=r_v3d0.scalarProduct(r_v3d1)
        det=s00*s11-s01*s01
        if det:
            v3d2=p_v3d.sub(v3d)
            s02=r_v3d0.scalarProduct(v3d2)
            s12=r_v3d1.scalarProduct(v3d2)
            return (float(s02*s11-s12*s01)/det,float(s00*s12-s02*s01)/det) 
        raise



# Algorithm:

def algorithm_CurvePoint(func,grad,v2d,delta):
    #Berechnet einen Punkt der Kurve entlang des "steilsten Weges"
    x=v2d.x
    y=v2d.y
    f=func(x,y)
    gx,gy=grad(x,y)
    cc=gx*gx+gy*gy
    if cc>0: f=-f/cc
    else: f=0
    newv2d=Vector2D(x+f*gx,y+f*gy)
    d=v2d.sub(newv2d).length()
    while delta<d:
        v2d=newv2d
        x=v2d.x
        y=v2d.y
        f=func(x,y)
        gx,gy=grad(x,y)
        cc=gx*gx+gy*gy
        if cc>0: f=-f/cc
        else: f=0
        newv2d=Vector2D(x+f*gx,y+f*gy)
        d=v2d.sub(newv2d).length()
    return newv2d

def algorithm_ImplicitCurvePoint(func,grad,v2d0,v2d1,maxpoints,step,delta):
    #Berechnet Kurvenpunkte einer impliziten Kurve mit Hilfe von algorithm_CurvePoint
    pointlist=[]
    startv2d=curve_point(func,grad,v2d0,delta)
    pointlist.append(startv2d)
    print "x0",startv2d.x
    print "y0",startv2d.y
    endv2d=curve_point(func,grad,v2d1,delta)
    gx,gy=grad(startv2d.x,startv2d.y)
    tangentv2d=Vector2D(-gy,gx)
    tangentlen=tangentv2d.length()
    startv2d=algorithm_CurvePoint(func,grad,startv2d.add(tangentv2d.scale(step/tangentlen)),delta)
    pointlist.append(startv2d)
    print "x0",startv2d.x
    print "y0",startv2d.y
    distance=step
    print "while"
    while (distance>step/2.0) and (len(pointlist)<maxpoints):
        gx,gy=grad(startv2d.x,startv2d.y)
        tangentv2d=Vector2D(-gy,gx)
        tangentlen=tangentv2d.length()
        startv2d=algorithm_CurvePoint(func,grad,startv2d.add(tangentv2d.scale(step/tangentlen)),delta)
        pointlist.append(startv2d)
        print "x",startv2d.x
        print "y",startv2d.y
        distance=startv2d.distance(endv2d)
    pointlist.append(endv2d)
    return pointlist
    