from cadmath import *
from sys import maxint


class ExtensionNode:
#Erweitert die Klasse Node3D vom Modul cadmath

    def __init__(self,v2d):
        #Init-Methode
        self.v2d=v2d #2D-Koordinaten des Punktes


class ExtensionEdge:
#Erweitert die Klasse Edge3D vom Modul cadmath

    def __init__(self):
        #Init-Methode
        self.intersectiondic={} #Dictionary: Schluessel sind Zeiger auf Facetten, Werte sind Node3D-Objekte 


class ExtensionFace:
#Erweitert die Klasse Face3D vom Modul cadmath

    def __init__(self,boundingbox,normalv3d,distance,orientation):
        #Init-Methode
        self.boundingbox=boundingbox #2D-Boundingbox
        self.normalv3d=normalv3d #Normalenvektor zur Facette mit Laenge 1.0
        self.distance=distance #Distanz vom Zentrum zur Facette
        self.orientation=orientation # orientation=1: Facette vor Zentrum, orientation=-1: Facette dahinter


minint=-maxint-1 #kleinster Integer-Wert
boundaryedgelist=[] #Kantenliste fuer Durchdringung

def intersectFaceEdge(face,edge,eps):
#Prueft ob die Kante die Facette im 2D schneidet
#Kanten der eigenen Facette werden als schneidend betrachtet
#Wenn ja, werden die Schnittverhaeltnisse bzgl. dem Kanten-Startknoten zurueckgegeben
#Wenn nein, wird ein leeres Array zurueckgegeben
    measurelist=[]
    startv2d=edge.startnode.extension.v2d
    endv2d=edge.endnode.extension.v2d
    x0=startv2d.x
    y0=startv2d.y
    x1=endv2d.x
    y1=endv2d.y
    nodelist=face.nodelist
    v2d=nodelist[-1].extension.v2d
    x2=v2d.x
    y2=v2d.y
    for node in nodelist:
        v2d=node.extension.v2d
        det=float((v2d.x-x2)*(y1-y0)-(v2d.y-y2)*(x1-x0))
        if abs(det)>eps:
            measure=((y2-y0)*(x1-x0)-(x2-x0)*(y1-y0))/det
            if -eps<=measure and measure<=1.0+eps:
                measure=((y2-y0)*(v2d.x-x0)-(x2-x0)*(v2d.y-y0))/det
                if not len(measurelist): measurelist.append(measure)
                elif abs(measure-measurelist[0])>eps:
                    measurelist.append(measure)
                    measurelist.sort()
                    if measurelist[0]>1.0-eps or measurelist[1]<eps: measurelist=[]
                    return measurelist #Kante schneidet oder ist innerhalb der Facette
        x2=v2d.x
        y2=v2d.y
    if len(measurelist)==1: measurelist.append(measurelist[0])
    return measurelist #Kante beruehrt in einem Punkt oder ist ausserhalb

def getBoundingBox(nodelist):
#Berechnet eine 2D-Boundingbox einer Knotenliste 
    boundingbox=[maxint,maxint,minint,minint]
    for node in nodelist:
        v2d=node.extension.v2d
        if v2d.x<boundingbox[0]: boundingbox[0]=v2d.x
        if v2d.y<boundingbox[1]: boundingbox[1]=v2d.y      
        if v2d.x>boundingbox[2]: boundingbox[2]=v2d.x
        if v2d.y>boundingbox[3]: boundingbox[3]=v2d.y
    return boundingbox

def testBoundingBoxEdge(boundingbox,edge):
#Prueft ob Kante die Boundingbox schneidet
    startv2d=edge.startnode.extension.v2d
    endv2d=edge.endnode.extension.v2d
    xmin=startv2d.x
    ymin=startv2d.y
    xmax=endv2d.x
    ymax=endv2d.y
    if xmin>xmax:
        tmp=xmax
        xmax=xmin
        xmin=tmp
    if ymin>ymax:
        tmp=ymax
        ymax=ymin
        ymin=tmp
    return xmax>boundingbox[0] and xmin<boundingbox[2] and ymax>boundingbox[1] and ymin<boundingbox[3]

def subIntervals(interval,imin,imax):
#Berechnet die Differenz zwischen der Intervall-Liste "interval" und dem  Intervall "imin,imax" 
#Die Rueckgabe ist eine neue Intervall-Liste
    newinterval=[]
    for i in interval:
        if i[1]<=imin or i[0]>=imax: newinterval.append(i)
        elif i[0]<imin:
            newinterval.append((i[0],imin))
            if i[1]>imax: newinterval.append((imax,i[1]))
        elif i[0]>=imin and i[1]>imax: newinterval.append((imax,i[1]))
    return newinterval

def loopOfEdges(graph,obj3d,boundary,eps,edgelist):
#Kantenschleife
    for edge in edgelist:
        if edge.extension==None: #Kante noch nicht besucht
            edge.extension=ExtensionEdge()
            interval=[(0.0,1.0)]
            startv2d=edge.startnode.extension.v2d
            endv2d=edge.endnode.extension.v2d
            directionv2d=endv2d.sub(startv2d)
            for face in obj3d.facelist:
                if face.extension<>None and testBoundingBoxEdge(face.extension.boundingbox,edge) and not edge in face.edgelist:
                    measurelist=intersectFaceEdge(face,edge,eps)
                    if measurelist<>[]: #Kante innerhalb oder Schnitt
                        startv3d=edge.startnode.v3d
                        endv3d=edge.endnode.v3d
                        v3d=endv3d.sub(startv3d)
                        mmin=max(0.0,measurelist[0])
                        mmax=min(1.0,measurelist[1])
                        normalv3d=face.extension.normalv3d
                        distance=face.extension.distance
                        orientation=face.extension.orientation
                        try:
                            r=float(normalv3d.scalarProduct(startv3d)-distance)/-normalv3d.scalarProduct(v3d)
                            if r>eps and r<1.0-eps:
                                intersectionv3d=function_linearCombine3D(startv3d,1,v3d,r)
                                intersectionv2d=graph.convertV3DToV2D(intersectionv3d)
                            else: intersectionv2d=None
                        except: intersectionv2d=None
                        startdis=(normalv3d.scalarProduct(startv3d)-face.extension.distance)*orientation
                        enddis=(normalv3d.scalarProduct(endv3d)-face.extension.distance)*orientation
                        if intersectionv2d==None:
                            if startdis<eps and enddis<=-eps or enddis<eps and startdis<=-eps: interval=subIntervals(interval,mmin,mmax)
                        else:
                            m=intersectionv2d.sub(startv2d).length()/directionv2d.length()
                            if m<mmax and m>mmin:
                                if startdis<0.0: interval=subIntervals(interval,mmin,m)
                                else: interval=subIntervals(interval,m,mmax)
                                if boundary:
                                    node=Node3D(intersectionv3d)
                                    node.extension=ExtensionNode(intersectionv2d)
                                    edge.extension.intersectiondic[face]=node
                            elif m<mmin and startdis>0.0 or m>mmax and startdis<0.0: interval=subIntervals(interval,mmin,mmax)
            for i in interval: graph.drawLine2D(startv2d.add(directionv2d.scale(i[0])),startv2d.add(directionv2d.scale(i[1])))
        
def notInBoundaryEdgeList(startnode,endnode):
#Ermittelt ob die Kante (endnode,startnode) nicht in der Kantenliste enthalten ist
    for edge in boundaryedgelist:
        if edge.startnode==endnode and edge.endnode==startnode: return 0 #enthalten
    return 1 #nicht enthalten

def hiddenLine(graph,obj3d,boundary=0,orientedfaces=1,eps=0.00001):
#Der Hiddenline Algorithmus
    centrev3d=graph.centrev3d
    for node in obj3d.nodelist: node.extension=None #Reset aller Knoten-Erweiterungen
    for edge in obj3d.edgelist: edge.extension=None #Reset aller Kanten-Erweiterungen
    for face in obj3d.facelist:
        face.extension=None
        nodelist=face.nodelist
        normalv3d,distance=function_GetPlaneEquation(nodelist[0].v3d,nodelist[1].v3d,nodelist[2].v3d)
        orientation=normalv3d.scalarProduct(centrev3d)-distance
        if not orientedfaces or orientation>=0.0:
            passed=1
            for node in nodelist:
                try:
                    v2d=graph.convertV3DToV2D(node.v3d)
                    node.extension=ExtensionNode(v2d)
                except: passed=0
            if passed:
                if orientation==abs(orientation): orientation=1
                else: orientation=-1
                face.extension=ExtensionFace(getBoundingBox(nodelist),normalv3d,distance,orientation)
    for face in obj3d.facelist:
        if face.extension<>None: loopOfEdges(graph,obj3d,boundary,eps,face.edgelist)
    if boundary:
        boundaryedgelist=[]
        for face in obj3d.facelist:
            if face.extension<>None:
                for edge in face.edgelist:
                    intersectiondic=edge.extension.intersectiondic
                    if len(intersectiondic)<>0:
                        for keyface in intersectiondic.keys():
                            for keyedge in keyface.edgelist:
                                if face in keyedge.extension.intersectiondic:
                                    startnode=intersectiondic[keyface]
                                    endnode=keyedge.extension.intersectiondic[face]
                                    if notInBoundaryEdgeList(startnode,endnode): boundaryedgelist.append(Edge3D(startnode,endnode))
                            for faceedge in face.edgelist:
                                if edge<>faceedge and keyface in faceedge.extension.intersectiondic:
                                    startnode=intersectiondic[keyface]
                                    endnode=faceedge.extension.intersectiondic[keyface]
                                    if notInBoundaryEdgeList(startnode,endnode): boundaryedgelist.append(Edge3D(startnode,endnode))
        loopOfEdges(graph,obj3d,0,eps,boundaryedgelist)   