from lib.cadmath import *
import math

class Graph:

    def __init__(self,canvas,drawWidth,drawHeight,offsetX=0,offsetY=0):
        self.canvas=canvas
        self.drawWidth=drawWidth
        self.drawHeight=drawHeight
        self.offsetX=offsetX
        self.offsetY=offsetY
        self.reset()

    def clear(self):
        canvas=self.canvas
        list=canvas.find_all()
        for item in list: canvas.delete(item)
        canvas.create_rectangle(self.offsetX,self.offsetY,self.drawWidth+self.offsetX,self.drawHeight+self.offsetY)

    def reset(self):
        self.setColor()
        self.setLineWidth()
        self.setStippled(0)
        self.setPointSize()        
        self.setOrigin(self.drawWidth/2,self.drawHeight/2)
        self.setScaling(1,1)
        self.useCentralProjection()

    def useParallelProjection(self):
        self.cp=0
        self.setAngles()

    def useCentralProjection(self):
        self.cp=1
        self.mainv3d=Vector3D() #Hauptpunkt
        self.distance=1000 #Distanz zwischen Hauptpunkt und Zentrum
        self.disappearance=0 #Verschwindungsebene: Abstand vom Zentrum in Richtung Hauptpunkt
        self.setAngles()

    def setMainPointC(self,x,y,z): self.setMainPoint(Vector3D(x,y,z)) #only used by CP

    def setMainPoint(self,mainv3d): #only used by CP
        self.mainv3d=mainv3d
        self.centrev3d=self.mainv3d.add(self.n0v3d.scale(self.distance))

    def setDistance(self,distance): #only used by CP
        self.distance=distance
        self.centrev3d=self.mainv3d.add(self.n0v3d.scale(distance))

    def setDisappearance(self,disappearance): #only used by CP
        self.disappearance=disappearance

    def setAngles(self,angle_u=0,angle_v=0): #used by CP and PP
        rad_u=angle_u*math.pi/180
        rad_v=angle_v*math.pi/180
        self.sin_u=sin_u=math.sin(rad_u)
        self.cos_u=cos_u=math.cos(rad_u)
        self.sin_v=sin_v=math.sin(rad_v)
        self.cos_v=cos_v=math.cos(rad_v)
        self.e1v3d=Vector3D(cos_u,0,-sin_u)
        self.e2v3d=Vector3D(-sin_u*sin_v,cos_v,-cos_u*sin_v)
        self.n0v3d=Vector3D(sin_u*cos_v,sin_v,cos_u*cos_v)
        if self.cp: self.centrev3d=self.mainv3d.add(self.n0v3d.scale(self.distance))

    def setOrigin(self,xorigin,yorigin):
        self.xoffset=xorigin+self.offsetX
        self.yoffset=self.drawHeight-yorigin+self.offsetY

    def setScaling(self,xscaling,yscaling):
        self.xscaling=xscaling
        self.yscaling=yscaling

    def setColor(self,color="black"): self.color=color 

    def setLineWidth(self,linewidth=1):
        if linewidth<1: linewidth=1
        self.linewidth=linewidth 

    def setStippled(self,stippled=1):
        self.stippled=self.getStippled(stippled)

    def getStippled(self,stippled):
        if stippled==0: return None
        else: return "gray50"

    def setPointSize(self,pointsize=1):
        if pointsize<1: self.pointsize=0
        else: self.pointsize=pointsize-1

# 2D functions

    def drawPoint2D(self,v2d,color=None,pointsize=None): self.drawPointC2D(v2d.x,v2d.y,color,pointsize)

    def drawPointC2D(self,x,y,color=None,pointsize=None):
        if color==None: color=self.color
        if pointsize==None: pointsize=self.pointsize
        elif pointsize<1: pointsize=0
        else: pointsize-=1
        x=self.xoffset+int(round(x*self.xscaling))
        y=self.yoffset-int(round(y*self.yscaling))
        self.canvas.create_oval((x-pointsize,y-pointsize),(x+pointsize,y+pointsize),fill=color)

    def drawArrow2D(self,v2d0,v2d1,color=None,linewidth=None,stippled=None,arrow="last"): self.drawLineC2D(v2d0.x,v2d0.y,v2d1.x,v2d1.y,color,linewidth,stippled,arrow)

    def drawArrowC2D(self,x0,y0,x1,y1,color=None,linewidth=None,stippled=None,arrow="last"): self.drawLineC2D(x0,y0,x1,y1,color,linewidth,stippled,arrow)        

    def drawLine2D(self,v2d0,v2d1,color=None,linewidth=None,stippled=None,arrow=None):
        self.drawLineC2D(v2d0.x,v2d0.y,v2d1.x,v2d1.y,color,linewidth,stippled,arrow)

    def drawLineC2D(self,x0,y0,x1,y1,color=None,linewidth=None,stippled=None,arrow=None):
        if color==None: color=self.color
        if linewidth==None: linewidth=self.linewidth
        if stippled==None: stippled=self.stippled
        else: stippled=self.getStippled(stippled)
        xs=self.xscaling
        ys=self.yscaling
        xo=self.xoffset
        yo=self.yoffset
        self.canvas.create_line(int(round(x0*xs))+xo,yo-int(round(y0*ys)),int(round(x1*xs))+xo,yo-int(round(y1*ys)),fill=color,width=linewidth,stipple=stippled,arrow=arrow)

    def drawCurve2D(self,v2dlist,close=0,color=None,linewidth=None,stippled=None):
        v2d0=v2dlist[0]
        for v2d1 in v2dlist[1:]:
            self.drawLine2D(v2d0,v2d1,color,linewidth,stippled)
            v2d0=v2d1
        if close: self.drawLine2D(v2d0,v2dlist[0],color,linewidth,stippled)

    def drawAxes2D(self,length,linewidth=None,stippled=None):
        self.drawArrowC2D(0,0,length,0,"red",linewidth,stippled)
        self.drawArrowC2D(0,0,0,length,"green",linewidth,stippled)

# 3D functions

    def transformV3DToE1E2N0(self,v3d):
        v3d=v3d.sub(self.mainv3d)
        return Vector3D(v3d.scalarProduct(self.e1v3d),v3d.scalarProduct(self.e2v3d),v3d.scalarProduct(self.n0v3d))

    def convertV3DToV2D(self,v3d):
    #Konvertiert 3D-Koordinaten in 2D-Koordinaten bzgl. der Bildebene
    #Erzeugt einen Fehler, wenn der Punkt hinterm Betrachter liegt
        if self.cp:
            v3d=self.transformV3DToE1E2N0(v3d)
            cc=1-v3d.z/float(self.distance)
            if cc>self.disappearance: return Vector2D(v3d.x/cc,v3d.y/cc)
            else: raise "The point "+v3d.toString()+"is behind the viewer"
        return Vector2D(v3d.y*self.cos_u-v3d.x*self.sin_u,v3d.z*self.cos_v-(v3d.x*self.cos_u+v3d.y*self.sin_u)*self.sin_v)

    def drawPoint3D(self,v3d,color=None,pointsize=None):
        v2d=self.convertV3DToV2D(v3d)
        self.drawPointC2D(v2d.x,v2d.y,color,pointsize)

    def drawPointC3D(self,x,y,z,color=None,pointsize=None):
        self.drawPoint3D(Vector3D(x,y,z),color,pointsize)
        
    def drawArrow3D(self,v3d0,v3d1,color=None,linewidth=None,stippled=None,arrow="last"):
        self.drawLine3D(v3d0,v3d1,color,linewidth,stippled,arrow)

    def drawArrowC3D(self,x0,y0,z0,x1,y1,z1,color=None,linewidth=None,stippled=None,arrow="last"):
        self.drawLine3D(Vector3D(x0,y0,z0),Vector3D(x1,y1,z1),color,linewidth,stippled,arrow)

    def drawLine3D(self,v3d0,v3d1,color=None,linewidth=None,stippled=None,arrow=None):
        self.drawLine2D(self.convertV3DToV2D(v3d0),self.convertV3DToV2D(v3d1),color,linewidth,stippled,arrow)

    def drawLineC3D(self,x0,y0,z0,x1,y1,z1,color=None,linewidth=None,stippled=None,arrow=None):
        self.drawLine3D(Vector3D(x0,y0,z0),Vector3D(x1,y1,z1),color,linewidth,stippled,arrow)

    def drawCurve3D(self,v3dlist,close=0,color=None,linewidth=None,stippled=None):
        v2dlist=[]
        for v3d in v3dlist: v2dlist.append(self.convertV3DToV2D(v3d))
        self.drawCurve2D(v2dlist,close,color,linewidth,stippled)

    def drawAxes3D(self,length,linewidth=None,stippled=None):
        self.drawArrowC3D(0,0,0,length,0,0,"red",linewidth,stippled)
        self.drawArrowC3D(0,0,0,0,length,0,"green",linewidth,stippled)
        self.drawArrowC3D(0,0,0,0,0,length,"blue",linewidth,stippled)