import os, sys, threading, time
if sys.platform=="win32":
	import _winreg
	try: import win32print
	except: pass

class PsPrint:

	def __init__(self,gui):
		self.inifile=gui.inifile
		self.canvas=gui.pntcanvas
		self.logic=gui.logic
		self.temppath=self.inifile.getProperty("temppath")
		if sys.platform=="win32":
			self.printer=self.inifile.getProperty("win32printer")
			self.printos="wprint"
		elif sys.platform[:5]=="linux" or sys.platform[:5]=="sunos":
			self.printer=self.inifile.getProperty("unixprinter")
			self.printos="xprint"
		else: self.printos=""

	def settings(self):
		inifile=self.inifile
		topmargin=inifile.getProperty("topmargin",type='f')
		bottommargin=inifile.getProperty("bottommargin",type='f')
		leftmargin=inifile.getProperty("leftmargin",type='f')
		rightmargin=inifile.getProperty("rightmargin",type='f')
		instancesperpage=inifile.getProperty("instancesperpage",type='i')
		papersize=inifile.getProperty("papersize")
		if papersize=="A3":
			paperwidth=842
			paperheight=1190
		elif papersize=="A5":
			paperwidth=420
			paperheight=595
		else: # papersize is A4
			paperwidth=595
			paperheight=842
		cm2px=28.35
		self.ph=paperheight-cm2px*(topmargin+bottommargin)
		self.pw=paperwidth-cm2px*(leftmargin+rightmargin)
		self.px=(paperwidth+cm2px*(leftmargin-rightmargin))/2
		self.py=(paperheight+cm2px*(bottommargin-topmargin))/2

	def print2File(self,filename):
		self.settings()
		canvas=self.canvas
		coords=self.logic.getCoords()
		width,height,mscname=coords.pop(0)
		filename,extension=os.path.splitext(filename)
		filename=filename+'.'
		for item in coords:
			file=open(filename+item[2]+extension,'w')
			file.write(canvas.postscript(x=item[0],y=item[1],width=width,height=height,pagex=self.px,pagey=self.py,pagewidth=self.pw,pageheight=self.ph))
			file.close()

	def print2Printer(self):
		self.settings()
		if self.printos=="wprint": self.print4Win32()
		elif self.printos=="xprint": self.print4Unix()
		else: raise Exception, "Unknown OS - Cannot detect printer"

	def print4Win32(self):
		canvas=self.canvas
		coords=self.logic.getCoords()
		width,height,mscname=coords.pop(0)
		mscname=mscname+'.'
		try:
			printer=win32print.OpenPrinter(self.win32printer)
			for item in coords:
				win32print.StartDocPrinter(printer,1,(mscname+item[2],None,None))
				win32print.WritePrinter(self.win32printer,canvas.postscript(x=item[0],y=item[1],width=width,height=height,pagex=self.px,pagey=self.py,pagewidth=self.pw,pageheight=self.ph))
				win32print.EndDocPrinter(self.win32printer)
			win32print.ClosePrinter(psprinter)
		except:
			epsfilespath=self.temppath+os.sep+str(int(time.time()))
			os.mkdir(epsfilespath)
			for item in coords:
				file=open(epsfilespath+os.sep+mscname+item[2]+".eps",'w')
				file.write(canvas.postscript(x=item[0],y=item[1],width=width,height=height,pagex=self.px,pagey=self.py,pagewidth=self.pw,pageheight=self.ph))
				file.close()
			GSviewThread(epsfilespath).start()

	def print4Unix(self):
		canvas=self.canvas
		coords=self.logic.getCoords()
		width,height=coords.pop(0)
		pipe=os.popen(self.unixprinter,'w')
		for item in coords: pipe.write(canvas.postscript(x=item[0],y=item[1],width=width,height=height,pagex=self.px,pagey=self.py,pagewidth=self.pw,pageheight=self.ph))


class GSviewThread(threading.Thread): # starts GSview in a Thread

	def __init__(self,epsfilepath): # initializes the class
		threading.Thread.__init__(self)
		self.epsfilepath=epsfilepath

	def run(self): # runs GSview with all files from folder(epsfilepath) and deletes files and folder
		epsfilepath=self.epsfilepath
		gspath=self.getGSPath()
		for filename in os.listdir(epsfilepath):
			currentfilepath=epsfilepath+os.sep+filename
			os.system(gspath+os.sep+"gsview32 /p"+currentfilepath)
			os.remove(currentfilepath)
		os.rmdir(epsfilepath)

	def getGSPath(self): # returns the path of GSview from the registry
		try:
			reg_handle=_winreg.ConnectRegistry(None,_winreg.HKEY_LOCAL_MACHINE)
			key=_winreg.OpenKey(reg_handle,"Software\\Ghostgum\\Gsview")
			count_key=_winreg.QueryInfoKey(key)[1]
			for i in range(count_key):
				key_val=_winreg.EnumValue(key,i)[0]
				gs_path=_winreg.QueryValueEx(key,key_val)[0]
				if os.path.exists(gs_path): break
			return gs_path+os.sep+"gsview"
		except: raise Exception,"Cannot detect GSview"