import string,tkFileDialog,os,code
from prg import *
from Tkinter import *

class GuiInteractiveInterpreter(code.InteractiveInterpreter):
    def __init__(self,*values,**paramvalues):
        code.InteractiveInterpreter.__init__(self,*values,**paramvalues)
        self.text=""
    def write(self,data):
        self.text+=data
    def read(self):
        text=self.text
        self.text=""
        return text

class HelpWindow(Toplevel):
    def __init__(self,gui):
        Toplevel.__init__(self,gui)
        self.wm_title("Manuals")
        self.minsize(300,300)
        menubar=Menu(self)
        self.themesmenu=themesmenu=Menu(menubar,tearoff=0)
        self.file=StringVar()
        path="."+os.sep+"hlp"
        for entry in os.listdir(path):
            file=path+os.sep+entry
            name,extension=os.path.splitext(entry)
            if os.path.isfile(file) and extension==".txt": themesmenu.add_radiobutton(label=name,command=self.displayFile,variable=self.file,value=file)
    	menubar.add_cascade(label="Themes",menu=themesmenu)
    	self.config(menu=menubar)
        textframe=Frame(self,relief=SUNKEN,bd=2)
        textframe.columnconfigure(0,weight=1)
        textframe.rowconfigure(0,weight=1)
        self.text=Text(textframe,wrap=NONE,height=0,width=0,bd=0,state=DISABLED,cursor=textframe.cget("cursor"),bg=textframe.cget("bg"))
        textscrollv=Scrollbar(textframe,orient=VERTICAL,command=self.text.yview)
        textscrollh=Scrollbar(textframe,orient=HORIZONTAL,command=self.text.xview)
        self.text.config(yscrollcommand=textscrollv.set,xscrollcommand=textscrollh.set)
        self.text.grid(row=0,column=0,sticky="nswe")
        textscrollv.grid(row=0,column=1,sticky="ns")
        textscrollh.grid(row=1,column=0,sticky="we")
        textframe.pack(fill=BOTH,expand=1)
        self.withdraw()
        gui.focus_set()
        self.gui=gui
    def reset(self):
        if self.state()=="withdrawn":
            self.geometry("%dx%d+%d+%d"%(300,300,self.gui.winfo_rootx(),self.gui.winfo_rooty()))
            self.themesmenu.invoke(0)
        self.deiconify()
        self.focus_set()
    def displayFile(self):
        self.text.config(state=NORMAL)
        self.text.delete("0.0",END)
        try:
            openfile=open(self.file.get(),'r')
            self.text.insert(END,openfile.read())
            openfile.close()
        except:
            self.text.insert(END,"Cant read from file!")
            self.text.config(state=DISABLED)
    def destroy(self): self.withdraw()

class PopupWindow(Toplevel):
    def __init__(self,gui,title,text,action=None,width=240,height=60):
        Toplevel.__init__(self,gui)
        self.action=action
        self.wm_title(title)
        self.transient(gui)
        self.grab_set()
        self.focus_set()
        self.resizable(0,0)
        self.gui=gui
        self.geometry("%dx%d+%d+%d"%(width,height,((int(self.gui.winfo_width())-width)>>1)+self.gui.winfo_rootx(),((int(self.gui.winfo_height())-height)>>1)+self.gui.winfo_rooty()))
        Label(self,text=text).pack(fill=BOTH,expand=1)
        frame=Frame(self)
        if action==None: Button(frame,text="OK",padx=10,command=self.destroy).pack()
        else:
            Button(frame,text="Yes",padx=10,command=self.doYes).pack(padx=2,side=LEFT)
            Button(frame,text="No",padx=10,command=self.doNo).pack(padx=2,side=RIGHT)
        frame.pack(side=BOTTOM)
        self.wait_window()
    def doYes(self):
        self.destroy()
        self.action()
    def doNo(self):
        self.gui.changed=0
        self.destroy()

class GuiText(Text):
    def __init__(self,gui,*values,**paramvalues):
        Text.__init__(self,*values,**paramvalues)
        self.bind("<Key>",self.format)
        self.config(font="Courier 10",height=0,width=0,bd=0)
        self.gui=gui
    def format(self,event):
        if not self.gui.changed&1 and not event.keysym in ["Up","Down","Left","Right","Shift_L","Control_L","Alt_L","Shift_R","Control_R","Alt_R","Caps_Lock","Escape","Prior","Next","Home","End","Print","Num_Lock","Scroll_Lock","F1","F2","F3","F4","F5","F6","F7","F8","F9","F10","F11","F12","Pause"]:
            self.gui.changed+=1
            self.gui.filemenu.entryconfig(2,state=NORMAL)
            self.gui.fileMenuSaveButton.config(state=NORMAL)
        if event.keysym=="Return":
            line=self.get(self.index(INSERT)+"linestart",self.index(INSERT)+"lineend")
            if len(line) and line[-1]==":":
                self.insert(INSERT,"\n\t")
            else: self.insert(INSERT,"\n")
            while len(line)>1 and line[0]=="\t":
                self.insert(INSERT,"\t")
                line=line[1:]
            self.see(INSERT)
            return "break"

class Gui(Tk):
    def __init__(self):
        Tk.__init__(self)
    	self.inifile=inifile=iniaccess.IniFileAccess("pycad.ini")
    	inifile.setProperty("_windowWidth",580)
    	inifile.setProperty("_windowHeight",420)
    	#-----------------------------------
        self.title("PyCAD")
        self.minsize(580,420)
        self.geometry("%dx%d" % (inifile.getProperty("windowWidth",type='i'),inifile.getProperty("windowHeight",type='i')))
    	self.menubar=menubar=Menu(self)
    	self.filemenu=filemenu=Menu(menubar,tearoff=0)
    	filemenu.add_command(label="New",command=self.fileMenuNew)
    	filemenu.add_command(label="Open",command=self.fileMenuOpen)
    	filemenu.add_command(label="Save",state=DISABLED,command=self.fileMenuSave)
    	filemenu.add_command(label="Save As",command=self.fileMenuSaveAs)
    	filemenu.add_separator()
    	filemenu.add_command(label="Quit",command=self.destroy)
    	menubar.add_cascade(label="File",menu=filemenu)
    	graphmenu=Menu(menubar,tearoff=0)
    	graphmenu.add_command(label="Clear",command=self.graphMenuClear)
    	graphmenu.add_separator()
    	graphmenu.add_command(label="Export PostScript",command=self.graphMenuExportPS)
    	menubar.add_cascade(label="Graph",menu=graphmenu)
    	consolemenu=Menu(menubar,tearoff=0)
    	consolemenu.add_command(label="Run",command=self.consoleMenuRun)
    	menubar.add_cascade(label="Console",menu=consolemenu)
    	helpmenu=Menu(menubar,tearoff=0)
    	helpmenu.add_command(label="Manuals",command=self.helpMenuManuals)
    	helpmenu.add_separator()
    	helpmenu.add_command(label="Info",command=self.helpMenuInfo)
    	menubar.add_cascade(label="Help",menu=helpmenu)
    	self.config(menu=menubar)
    	#-----------------------------------
        self.newLogo=PhotoImage(data=base64pic.getNewLogo())
        self.openLogo=PhotoImage(data=base64pic.getOpenLogo())
        self.saveLogo=PhotoImage(data=base64pic.getSaveLogo())
        self.saveAsLogo=PhotoImage(data=base64pic.getSaveAsLogo())
        self.clearLogo=PhotoImage(data=base64pic.getClearLogo())
        self.runLogo=PhotoImage(data=base64pic.getRunLogo())
        self.helpLogo=PhotoImage(data=base64pic.getHelpLogo())
    	#-----------------------------------
    	topframe=Frame(self,relief=GROOVE,bd=2)
    	Button(topframe,image=self.newLogo,command=self.fileMenuNew).pack(side=LEFT)
    	Button(topframe,image=self.openLogo,command=self.fileMenuOpen).pack(side=LEFT)
        self.fileMenuSaveButton=Button(topframe,image=self.saveLogo,command=self.fileMenuSave,state=DISABLED)
        self.fileMenuSaveButton.pack(side=LEFT)
        Button(topframe,image=self.saveAsLogo,command=self.fileMenuSaveAs).pack(side=LEFT)
        Frame(topframe,width=5).pack(side=LEFT)
        Button(topframe,image=self.clearLogo,command=self.graphMenuClear).pack(side=LEFT)
        Frame(topframe,width=5).pack(side=LEFT)
        Button(topframe,image=self.runLogo,command=self.consoleMenuRun).pack(side=LEFT)
        Frame(topframe,width=5).pack(side=LEFT)
        Button(topframe,image=self.helpLogo,command=self.helpMenuManuals).pack(side=LEFT)
    	topframe.pack(fill=X,side=TOP)
    	#-----------------------------------
        leftframe=Frame(self)
        lefttopframe=Frame(leftframe,relief=SUNKEN,bd=2)
        self.canvas=Canvas(lefttopframe,height=321,width=321,highlightthickness=0)
        self.canvas.pack()
        lefttopframe.pack()
        leftbottomframe=Frame(leftframe,relief=SUNKEN,bd=2)
        leftbottomframe.columnconfigure(0,weight=1)
        leftbottomframe.rowconfigure(0,weight=1)
        self.statustext=Text(leftbottomframe,wrap=NONE,height=0,width=0,bd=0,bg=leftbottomframe.cget("bg"),cursor=leftbottomframe.cget("cursor"))
        self.statustext.insert(END,"Drawing area: 300x300 pixel")
        statustextscrollv=Scrollbar(leftbottomframe,orient=VERTICAL,command=self.statustext.yview)
        statustextscrollh=Scrollbar(leftbottomframe,orient=HORIZONTAL,command=self.statustext.xview)
        self.statustext.config(yscrollcommand=statustextscrollv.set,xscrollcommand=statustextscrollh.set,state=DISABLED)
        self.statustext.grid(row=0,column=0,sticky="nswe")
        statustextscrollv.grid(row=0,column=1,sticky="ns")
        statustextscrollh.grid(row=1,column=0,sticky="we")
        leftbottomframe.pack(fill=BOTH,side=BOTTOM,expand=1)
        leftframe.pack(fill=BOTH,side=LEFT)
    	#-----------------------------------
        self.rightframe=rightframe=Frame(self,relief=SUNKEN,bd=2)
        rightframe.columnconfigure(0,weight=1)
        rightframe.rowconfigure(0,weight=1)
        self.text=GuiText(self,rightframe,wrap=NONE,setgrid=0)
        textscrollv=Scrollbar(rightframe,orient=VERTICAL,command=self.text.yview)
        textscrollh=Scrollbar(rightframe,orient=HORIZONTAL,command=self.text.xview)
        self.text.config(yscrollcommand=textscrollv.set,xscrollcommand=textscrollh.set)
        self.text.grid(row=0,column=0,sticky="nswe")
        textscrollv.grid(row=0,column=1,sticky="ns")
        textscrollh.grid(row=1,column=0,sticky="we")
        rightframe.pack(fill=BOTH,expand=1,side=RIGHT)
    	#-----------------------------------
        self.helpWindow=HelpWindow(self) # Hilfe-Fenster
    	self.file=""
        self.changed=0
        self.canvas.create_rectangle(10,10,310,310)
        self.wait_visibility()
    	self.initGuiII()
    def initGuiII(self): # Connectors to the GUI
        local={}
        local['graph']=cadgraph.Graph(self.canvas,300,300,10,10)
        local['io']=cadio.IO(self)
        self.guiii=GuiInteractiveInterpreter(local)
        if len(sys.argv)<2: self.fileMenuNew()
        else: self.fileMenuOpen(sys.argv[1])
    def graphMenuClear(self):
        canvas=self.canvas
        list=canvas.find_all()
        for item in list: canvas.delete(item)
        canvas.create_rectangle(10,10,310,310)
    def graphMenuExportPS(self):
        file="unnamed.ps"
        if os.path.exists(file):
            counter=0
            file="unnamed"+str(counter)+".ps"
            while os.path.exists(file):
                counter+=1
                file="unnamed"+str(counter)+".ps"
        file=os.path.normpath(tkFileDialog.asksaveasfilename(parent=self,initialfile=file,title="Save As",filetypes=[("PostScript","*.ps")],defaultextension=".ps").encode('latin-1'))
        if file<>"" and file<>".":
            try: self.canvas.postscript(x=10,y=10,width=300,height=300,file=file)
            except: PopupWindow(self,"Error","Can't write to file!")
    def consoleMenuRun(self):
        self.print2StatusText("\nRunning...")
        self.guiii.runcode(self.text.get("0.0",self.text.index(END)+"lineend"))
        text=self.guiii.read()
        if text<>"":
            self.print2StatusText(text[:-1])
            #Popup(self,"Runtime error",text)
            try: # springe zu fehler
                self.text.mark_set(INSERT,string.split(string.split(text,"line ",1)[1],",",1)[0]+".0")
                self.text.yview(self.text.index(INSERT))
            except: pass
        else: self.print2StatusText("OK")
    def saveLast(self):
        if self.changed&1:
            if self.changed&2: PopupWindow(self,"Question","Save Changes?",action=self.fileMenuSave)
            else: PopupWindow(self,"Question","Save Changes?",action=self.fileMenuSaveAs)
        return not self.changed&1
    def destroy(self):
        self.saveLast()
        inifile=self.inifile
        windowWidth,windowHeight=string.split(string.split(self.geometry(),'+',1)[0],'x',2)
        inifile.setProperty("windowWidth",windowWidth)
        inifile.setProperty("windowHeight",windowHeight)
        inifile.flushProperties()
        Tk.destroy(self)
    def saveDisabled(self,file):
        self.filemenu.entryconfig(2,state=DISABLED)
        self.fileMenuSaveButton.config(state=DISABLED)
        self.title("PyCAD - "+file)
        self.file=file
    def fileMenuNew(self):
        if self.saveLast():
            self.text.delete("0.0",END)
            file="unnamed.xpy"
            if os.path.exists(file):
                counter=0
                file="unnamed"+str(counter)+".xpy"
                while os.path.exists(file):
                    counter+=1
                    file="unnamed"+str(counter)+".xpy"
            self.saveDisabled(file)
            self.changed=0
    def fileMenuOpen(self,file=None):
        if self.saveLast():
            self.fileMenuNew()
            if (file==None): file=os.path.normpath(tkFileDialog.askopenfilename(parent=self,title="Open",filetypes=[("Python File","*.xpy"),("All Files","*.*")]).encode('latin-1'))
            if file<>"" and file<>".":
                try:
                    openfile=open(file,'r')
                    self.text.insert(END,openfile.read()[:-1])
                    openfile.close()
                    self.saveDisabled(file)
                    self.changed=2
                except: PopupWindow(self,"Error","Can't read from file!")
    def fileMenuSave(self,file=None):
        if file==None:
            file=self.file
            if self.changed==1: file=os.path.normpath(tkFileDialog.asksaveasfilename(parent=self,title="Save As",initialfile=file,filetypes=[("Python File","*.xpy"),("All Files","*.*")],defaultextension=".xpy").encode('latin-1'))
        if file<>"" and file<>".":
            try:
                openfile=open(file,'w')
                openfile.write(self.text.get("0.0",END))
                openfile.close()
                self.saveDisabled(file)
                self.changed=2
            except: PopupWindow(self,"Error","Can't write to file!")
    def fileMenuSaveAs(self,file=None):
        if file==None: file=self.file
        file=os.path.normpath(tkFileDialog.asksaveasfilename(parent=self,title="Save As",initialfile=file,filetypes=[("Python File","*.xpy"),("All Files","*.*")],defaultextension=".xpy").encode('latin-1'))
        self.fileMenuSave(file)
    def print2StatusText(self,text):
        self.statustext.config(state=NORMAL)
        self.statustext.insert(END,"\n"+text)
        self.statustext.yview(END)
        self.statustext.config(state=DISABLED)
    def helpMenuManuals(self): self.helpWindow.reset()
    def helpMenuInfo(self): PopupWindow(self,"Info","Build: (2002)0606")
        
if __name__ == "__main__":
    Gui().mainloop()